@@ -1,9 +1,8 @@
 /*-------------------------------------------------------------------------
- *
  * pgut.h
  *
- * Copyright (c) 2009-2010, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
- *
+ * Portions Copyright (c) 2008-2011, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Portions Copyright (c) 2011, Itagaki Takahiro
  *-------------------------------------------------------------------------
  */
 
@@ -11,23 +10,16 @@
 #define PGUT_H
 
 #include "c.h"
+#include <assert.h>
+
+#ifndef WIN32
+#include <sys/time.h>
+#include <unistd.h>
+#endif
+
 #include "libpq-fe.h"
 #include "pqexpbuffer.h"
-
-#include <assert.h>
-#include <sys/time.h>
-
-#if !defined(C_H) && !defined(__cplusplus)
-#ifndef bool
-typedef char bool;
-#endif
-#ifndef true
-#define true	((bool) 1)
-#endif
-#ifndef false
-#define false	((bool) 0)
-#endif
-#endif
+#include "utils/elog.h"
 
 #define INFINITE_STR		"INFINITE"
 
@@ -38,40 +30,6 @@ typedef enum YesNo
 	YES
 } YesNo;
 
-typedef enum pgut_optsrc
-{
-	SOURCE_DEFAULT,
-	SOURCE_ENV,
-	SOURCE_FILE,
-	SOURCE_CMDLINE,
-	SOURCE_CONST
-} pgut_optsrc;
-
-/*
- * type:
- *	b: bool (true)
- *	B: bool (false)
- *  f: pgut_optfn
- *	i: 32bit signed integer
- *	u: 32bit unsigned integer
- *	I: 64bit signed integer
- *	U: 64bit unsigned integer
- *	s: string
- *  t: time_t
- *	y: YesNo (YES)
- *	Y: YesNo (NO)
- */
-typedef struct pgut_option
-{
-	char		type;
-	char		sname;		/* short name */
-	const char *lname;		/* long name */
-	void	   *var;		/* pointer to variable */
-	pgut_optsrc	allowed;	/* allowed source */
-	pgut_optsrc	source;		/* actual source */
-} pgut_option;
-
-typedef void (*pgut_optfn) (pgut_option *opt, const char *arg);
 typedef void (*pgut_atexit_callback)(bool fatal, void *userdata);
 
 /*
@@ -82,52 +40,33 @@ extern const char  *PROGRAM_VERSION;
 extern const char  *PROGRAM_URL;
 extern const char  *PROGRAM_EMAIL;
 
-extern void	pgut_help(bool details);
-
 /*
  * pgut framework variables and functions
  */
-extern const char  *dbname;
-extern const char  *host;
-extern const char  *port;
-extern const char  *username;
-extern char		   *password;
-extern bool			debug;
-extern bool			quiet;
+extern bool		interrupted;
+extern int		pgut_log_level;
+extern int		pgut_abort_level;
+extern bool		pgut_echo;	
 
-#ifndef PGUT_NO_PROMPT
-extern YesNo	prompt_password;
-#endif
-
-extern PGconn	   *connection;
-extern bool			interrupted;
-
-extern void help(bool details);
-extern int pgut_getopt(int argc, char **argv, pgut_option options[]);
-extern void pgut_readopt(const char *path, pgut_option options[], int elevel);
-extern void pgut_setopt(pgut_option *opt, const char *optarg, pgut_optsrc src);
-extern bool pgut_keyeq(const char *lhs, const char *rhs);
+extern void pgut_init(int argc, char **argv);
 extern void pgut_atexit_push(pgut_atexit_callback callback, void *userdata);
 extern void pgut_atexit_pop(pgut_atexit_callback callback, void *userdata);
+extern void pgut_putenv(const char *key, const char *value);
 
 /*
  * Database connections
  */
-extern PGconn *pgut_connect(int elevel);
-extern PGconn *pgut_connectdb(const char *conninfo, int elevel);
+extern PGconn *pgut_connect(const char *info, YesNo prompt, int elevel);
 extern void pgut_disconnect(PGconn *conn);
-extern PGresult *pgut_execute(PGconn* conn, const char *query, int nParams, const char **params, int elevel);
-extern ExecStatusType pgut_command(PGconn* conn, const char *query, int nParams, const char **params, int elevel);
-extern bool pgut_send(PGconn* conn, const char *query, int nParams, const char **params, int elevel);
+extern void pgut_disconnect_all(void);
+extern PGresult *pgut_execute(PGconn* conn, const char *query, int nParams, const char **params);
+PGresult *pgut_execute_elevel(PGconn* conn, const char *query, int nParams, const char **params, int elevel);
+extern ExecStatusType pgut_command(PGconn* conn, const char *query, int nParams, const char **params);
+extern bool pgut_commit(PGconn *conn);
+extern void pgut_rollback(PGconn *conn);
+extern bool pgut_send(PGconn* conn, const char *query, int nParams, const char **params);
 extern int pgut_wait(int num, PGconn *connections[], struct timeval *timeout);
 
-extern PGconn *reconnect_elevel(int elevel);
-extern void reconnect(void);
-extern void disconnect(void);
-extern PGresult *execute_elevel(const char *query, int nParams, const char **params, int elevel);
-extern PGresult *execute(const char *query, int nParams, const char **params);
-extern void command(const char *query, int nParams, const char **params);
-
 /*
  * memory allocators
  */
@@ -139,37 +78,38 @@ extern char *strdup_trim(const char *str);
 
 #define pgut_new(type)			((type *) pgut_malloc(sizeof(type)))
 #define pgut_newarray(type, n)	((type *) pgut_malloc(sizeof(type) * (n)))
+#define pgut_newvar(type, m, n)	((type *) pgut_malloc(offsetof(type, m) + (n)))
 
 /*
  * file operations
  */
-extern FILE *pgut_fopen(const char *path, const char *mode, bool missing_ok);
-extern void pgut_mkdir(const char *path);
+extern FILE *pgut_fopen(const char *path, const char *mode);
+extern bool pgut_mkdir(const char *path);
 
 /*
  * elog
  */
-#define LOG			(-4)
-#define INFO		(-3)
-#define NOTICE		(-2)
-#define WARNING		(-1)
-#define HELP		1
-#define ERROR		2
-#define FATAL		3
-#define PANIC		4
-
-#define ERROR_SYSTEM			10	/* I/O or system error */
-#define ERROR_NOMEM				11	/* memory exhausted */
-#define ERROR_ARGS				12	/* some configurations are invalid */
-#define ERROR_INTERRUPTED		13	/* interrupted by signal */
-#define ERROR_PG_COMMAND		14	/* PostgreSQL query or command error */
-#define ERROR_PG_CONNECT		15	/* PostgreSQL connection error */
+#define E_PG_CONNECT	(-1)	/* PostgreSQL connection error */
+#define E_PG_COMMAND	(-2)	/* PostgreSQL query or command error */
 
 #undef elog
-extern void
-elog(int elevel, const char *fmt, ...)
-__attribute__((format(printf, 2, 3)));
+#undef ereport
+#define ereport(elevel, rest) \
+	(pgut_errstart(elevel) ? (pgut_errfinish rest) : (void) 0)
 
+extern void elog(int elevel, const char *fmt, ...)
+__attribute__((format(printf, 2, 3)));
+extern const char *format_elevel(int elevel);
+extern int parse_elevel(const char *value);
+extern int errcode_errno(void);
+extern bool log_required(int elevel, int log_min_level);
+extern bool pgut_errstart(int elevel);
+extern void pgut_errfinish(int dummy, ...);
+extern void pgut_error(int elevel, int code, const char *msg, const char *detail);
+
+/*
+ * CHECK_FOR_INTERRUPTS
+ */
 #undef CHECK_FOR_INTERRUPTS
 extern void CHECK_FOR_INTERRUPTS(void);
 
@@ -209,6 +149,7 @@ extern void CHECK_FOR_INTERRUPTS(void);
 #define appendStringInfoChar	appendPQExpBufferChar
 #define appendBinaryStringInfo	appendBinaryPQExpBuffer
 
+extern bool appendStringInfoVA(StringInfo str, const char *fmt, va_list args);
 extern int appendStringInfoFile(StringInfo str, FILE *fp);
 extern int appendStringInfoFd(StringInfo str, int fd);
 
