@@ -4,12 +4,12 @@ SET client_min_messages = warning;
 --
 CREATE TABLE tbl_cluster (
 	col1 int,
-	col2 timestamp,
+	"time" timestamp,
 	","")" text,
 	PRIMARY KEY (","")", col1) WITH (fillfactor = 75)
 ) WITH (fillfactor = 70);
 
-CREATE INDEX ","") cluster" ON tbl_cluster (col2, length(","")"), ","")" text_pattern_ops) WITH (fillfactor = 75);
+CREATE INDEX ","") cluster" ON tbl_cluster ("time", length(","")"), ","")" text_pattern_ops) WITH (fillfactor = 75);
 ALTER TABLE tbl_cluster CLUSTER ON ","") cluster";
 
 CREATE TABLE tbl_only_pkey (
@@ -80,6 +80,8 @@ ALTER TABLE tbl_with_dropped_column DROP COLUMN d1;
 ALTER TABLE tbl_with_dropped_column DROP COLUMN d2;
 ALTER TABLE tbl_with_dropped_column DROP COLUMN d3;
 ALTER TABLE tbl_with_dropped_column ADD COLUMN c3 text;
+CREATE VIEW view_for_dropped_column AS
+	SELECT * FROM tbl_with_dropped_column;
 
 INSERT INTO tbl_with_dropped_toast VALUES(1, 10, 'abc');
 INSERT INTO tbl_with_dropped_toast VALUES(2, 20, sqrt(2::numeric(1000,999))::text || sqrt(3::numeric(1000,999))::text);
@@ -89,6 +91,7 @@ ALTER TABLE tbl_with_dropped_toast DROP COLUMN t;
 --
 
 SELECT * FROM tbl_with_dropped_column;
+SELECT * FROM view_for_dropped_column;
 SELECT * FROM tbl_with_dropped_toast;
 
 --
@@ -110,12 +113,23 @@ SELECT * FROM tbl_with_dropped_toast;
 \d tbl_with_dropped_column
 \d tbl_with_dropped_toast
 
-SELECT col1, to_char(col2, 'YYYY-MM-DD HH24:MI:SS'), ","")" FROM tbl_cluster;
+SELECT col1, to_char("time", 'YYYY-MM-DD HH24:MI:SS'), ","")" FROM tbl_cluster;
 SELECT * FROM tbl_only_ckey ORDER BY 1;
 SELECT * FROM tbl_only_pkey ORDER BY 1;
 SELECT * FROM tbl_gistkey ORDER BY 1;
+
+SET enable_seqscan = on;
+SET enable_indexscan = off;
 SELECT * FROM tbl_with_dropped_column;
+SELECT * FROM view_for_dropped_column;
 SELECT * FROM tbl_with_dropped_toast;
+SET enable_seqscan = off;
+SET enable_indexscan = on;
+SELECT * FROM tbl_with_dropped_column;
+SELECT * FROM view_for_dropped_column;
+SELECT * FROM tbl_with_dropped_toast;
+RESET enable_seqscan;
+RESET enable_indexscan;
 
 --
 -- check broken links or orphan toast relations
@@ -132,12 +146,17 @@ SELECT oid, relname
    AND reltoastrelid NOT IN (SELECT oid FROM pg_class WHERE relkind = 't');
 
 --
--- clean up
+-- NOT NULL UNIQUE
 --
-
-DROP TABLE tbl_cluster;
-DROP TABLE tbl_only_pkey;
-DROP TABLE tbl_only_ckey;
-DROP TABLE tbl_gistkey;
-DROP TABLE tbl_with_dropped_column;
-RESET client_min_messages;
+CREATE TABLE tbl_nn    (col1 int NOT NULL, col2 int NOT NULL);
+CREATE TABLE tbl_uk    (col1 int NOT NULL, col2 int         , UNIQUE(col1, col2));
+CREATE TABLE tbl_nn_uk (col1 int NOT NULL, col2 int NOT NULL, UNIQUE(col1, col2));
+CREATE TABLE tbl_pk_uk (col1 int NOT NULL, col2 int NOT NULL, PRIMARY KEY(col1, col2), UNIQUE(col2, col1));
+\! pg_reorg --dbname=contrib_regression --no-order --table=tbl_nn
+-- => ERROR
+\! pg_reorg --dbname=contrib_regression --no-order --table=tbl_uk
+-- => ERROR
+\! pg_reorg --dbname=contrib_regression --no-order --table=tbl_nn_uk
+-- => OK
+\! pg_reorg --dbname=contrib_regression --no-order --table=tbl_pk_uk
+-- => OK
